/*

File Name: absmap.js

------------------------------------------------

AbsoluteStructure Map of The Urantia Book       last edit 2010-03-28 trb

2009-12-02 - added 51:7 to list of multiline section titles
2010-01-02 - corrected absLineNumberInSectionInPaper(paperNumber, absSectionNumber, lineNumberInAbsSection)
2010-02-04 - added 148:42 to list of multiline section titles
2010-03-28 - changed absNumberOfSublinesInLine(paperNumber, lineNumber) to return 2 for line 1 in all papers except Paper 0.

Copyright (C) 2007, 2009 Troy R. Bishop.  All rights reserved.

LICENSE --------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
---------------------------------------------------------------------------------

This object contains the following public methods:

   absLineNumberInExemplarSection (paperNumber, lineNumber)
   absLineNumberInAbsSection (paperNumber, lineNumber)
   absLineType  (paperNumber, lineNumber)
   absNumberOfLinesInAbsSection (paperNumber, absSectionNumber)
   absNumberOfLinesInExemplarSection (paperNumber, exemplarSectionNumber)
   absNumberOfSublinesInLine (paperNumber, lineNumber)
   absNumberOfSublinesInLineBookwideArray ()
   absSectionNumberOfLine (paperNumber, lineNumber)
   absLineNumberInSectionInPaper(paperNumber, absSectionNumber, lineNumberInAbsSection)
   exemplarSectionNumberToAbsSectionNumber (paperNumber, exemplarSectionNumber)
   absSectionNumberToExemplarSectionNumber = function (paperNumber, absSectionNumber)
   expandAbsDataForPaperWhereNecessary (paperNumber)
   lineIsAcknowledgmentHeading (paperNumber, lineNumberInPaper)
   lineIsPaperTitle (lineNumberInPaper)
   lineIsSectionTitle(paperNumber, lineNumberInPaper)

   lineNumber, sectionNumber, and paperNumber are purely physical and are unmapped.
   lineNumber is one-based and unmapped.
   sectionNumber is zero-based and unmapped.
   paperNumber is zero-based and unmapped.

-------------------------------------------------
*/

//this is the absMap constructor
function absMap ()
   {

//   BEGIN (PRIVATE) PROPERTIES

   this.vcurrentAbsPaperNumber = null;

//Keyed on paperNumber:lineNumberInPaper (holding the section title) joined by a colon.
   this.vabsNumberOfSublinesInLineBookwideArray = 
      {
      "42:14" : 2,
      "42:118" : 2,

      "51:38" : 2,

      "57:67" : 2,
      "57:78" : 3,

      "59:11" : 2,
      "59:32" : 2,
      "59:46" : 2,
      "59:59" : 3,
      "59:78" : 3,
      "59:102" : 3,

      "60:35" : 3,

      "61:5" : 2,
      "61:20" : 2,
      "61:34" : 2,
      "61:50" : 2,

      "148:42" : 2
      };

      var absLineTypeNameArray =
         {
         "papertitle" : "papertitle",
         "sectiontitle" : "sectiontitle",
         "acknowledgmentheading" : "acknowledgmentheading",
         "paragraph" : "paragraph"
         };

   var absNumberOfSublinesInLinePaperwideArray = [];

//------------------------------------------------

//     END (PRIVATE) PROPERTIES


//     BEGIN METHODS

   this.expandAbsDataForPaper = function (paperNumber)
      {
      if (paperNumber == this.vcurrentAbsPaperNumber)
         {
         return;
         }
      var sectionIndex = null;
      var lineIndex = null;
      var zeroIndex = null;
      var sublinesNumber = null;
      var sublinesQuantity = null;
      var splitKeyArray = null;
      var displayString = "";
      var sectionLineIndex = null;
      var lowSectionLineNumber = null;
      var highSectionLineNumber = null;
      var currentLineNumberInPaper = null;
      var lineNumberInSection = null;

      this.vcurrentAbsPaperNumber = paperNumber;
      this.sectionArray[0].absNumberOfSublinesInSectionTitleLine = 0;
      this.sectionArray[0].absSectionNumber = 0;

      for (sectionIndex = 1; sectionIndex <= this.vnumberOfSectionsInPaper; sectionIndex++)
         {
         //fill absSectionNumber property in sectionArray elements
         if (this.vcurrentAbsPaperNumber == 139 && sectionIndex > 9)
            {
           //adjustment because of original labeling anomaly that skips labsSection 10 in Paper 139
            this.sectionArray[sectionIndex].absSectionNumber = sectionIndex + 1;
            }
         else
            {
            this.sectionArray[sectionIndex].absSectionNumber = sectionIndex;
            }
      //initialize absNumberOfSublinesInLine property in sectionArray elements to 0
         this.sectionArray[sectionIndex].absNumberOfSublinesInSectionTitleLine = 0;
         }

      //set lineType for all lines to "paragraph" and absNumberOfSublinesInLine to 0
      for (lineIndex = 1; lineIndex <= this.vnumberOfLinesInPaper; lineIndex++)
         {
         this.lineArray[lineIndex] = new Array();
         this.lineArray[lineIndex].absLineType = "paragraph";
         this.lineArray[lineIndex].absNumberOfSublinesInLine = 0;
         }

      //set line type for first line to "paperTitle"
      this.lineArray[1].AbslineType = "papertitle";

      //set line type for lines that contain section titles to "sectionTitle"
      for (sectionIndex = 0; sectionIndex < this.vnumberOfSectionsInPaper; sectionIndex++)
         {
         if (sectionIndex != 0)
            {
            this.lineArray[this.sectionArray[sectionIndex].sectionTitleLineNumberInPaper].absLineType = "sectiontitle";
            }

         lowSectionLineNumber = this.sectionArray[sectionIndex].lowSectionLineNumberInPaper;
         highSectionLineNumber = this.sectionArray[sectionIndex].highSectionLineNumberInPaper;

         lineNumberInSection = 0;
         //in lineArray, for each line set sectionNumberOfLine and labelSectionNumberOfLine
         for (sectionLineIndex = lowSectionLineNumber; sectionLineIndex <= highSectionLineNumber; sectionLineIndex++)
            {
            currentLineNumberInPaper = this.sectionArray[sectionIndex].sectionTitleLineNumberInPaper + lineNumberInSection;

            //fill absSectionNumberOfLine property for all lineArray elements in section
            if (paperNumber != 139 || currentLineNumberInPaper < 121)
               {
               this.lineArray[currentLineNumberInPaper].absSectionNumberOfLine = sectionIndex;
               }
            else
               {
               //increase absSectionNumberOfLine by 1 because section 10 is skipped in abs structure
               this.lineArray[currentLineNumberInPaper].absSectionNumberOfLine = sectionIndex + 1;
               }
            this.lineArray[currentLineNumberInPaper].sectionNumberOfLine = sectionIndex;
//            if (paperNumber == 0)
//               {
               //fill in lineNumberInSection and absLineNumberInSection for Paper 0
//               if (currentLineNumberInPaper != 180)
//                  {
//                  lineNumberInSection++;
//                  }
//               if (currentLineNumberInPaper == 181)
//                  {
                  //line no. 181 in Paper 0 is not counted in calculation of absLineNumberInSection
//                  this.lineArray[currentLineNumberInPaper].absLineNumberInSection = -1;
//                  }
//               else
//                  {
//                  this.lineArray[currentLineNumberInPaper].absLineNumberInSection = lineNumberInSection;
//                  }
//               }
//            else
//               {
               this.lineArray[currentLineNumberInPaper].absLineNumberInSection = lineNumberInSection;
               lineNumberInSection++;
//               }
            }
         }

      if (this.vcurrentAbsPaperNumber == 0)
         {
         //set line type for the sole acknowledgment heading to "acknowledgment heading"
         this.lineArray[181].absLineType = "acknowledgmentheading";
         }

      //fill number of sublines property in sectionArray elements and lineArray elements
      for (sublinesNumber in this.vabsNumberOfSublinesInLineBookwideArray)
         {
         splitKeyArray = sublinesNumber.split(":");
         if (splitKeyArray[0] == this.vcurrentAbsPaperNumber)
            {
            splitLineNumber = splitKeyArray[1];
            sublinesQuantity = this.vabsNumberOfSublinesInLineBookwideArray[sublinesNumber];
            this.lineArray[splitLineNumber].absNumberOfSublinesInLine = sublinesQuantity;
            this.sectionArray[this.lineArray[splitLineNumber].sectionNumberOfLine].absNumberOfSublinesInSectionTitleLine = sublinesQuantity;
            }
         }
      }

   this.absLineNumberInExemplarSection = function (paperNumber, lineNumber)
      {
      if (!this.paperNumberAndLineNumberAreValid(paperNumber, lineNumber))
         {
         return -1;
         }
      this.expandExemplarDataForPaperWhereNecessary(paperNumber);
      return this.lineArray[lineNumber].absLineNumberInSection;
      }

   this.absLineNumberInAbsSection = function (paperNumber, lineNumber)
      {
      if (!this.paperNumberAndLineNumberAreValid(paperNumber, lineNumber))
         {
         return -1;
         }
      this.expandAbsDataForPaperWhereNecessary(paperNumber);
      return this.lineArray[lineNumber].absLineNumberInSection;
      }

   this.absLineType = function (paperNumber, lineNumber)
      {
      if (!this.paperNumberAndLineNumberAreValid(paperNumber, lineNumber))
         {
         return -1;
         }
      this.expandExemplarDataForPaperWhereNecessary(paperNumber);
      if (paperNumber == 0 && lineNumber == 181)
         {
         return "acknowledgmentheading";
         }
      else
         {
         return this.lineType(paperNumber, lineNumber);
         }
      }

   this.absNumberOfLinesInAbsSection = function (paperNumber, absSectionNumber)
      {
      if (paperNumber == 139 && absSectionNumber > 9)
         {
         return this.absNumberOfLinesInExemplarSection(paperNumber, absSectionNumber - 1);
         }
      else
         {
         return this.absNumberOfLinesInExemplarSection(paperNumber, absSectionNumber);
         }
      }

   this.absNumberOfLinesInExemplarSection = function (paperNumber, exemplarSectionNumber)
      {
      if (paperNumber < 0 || paperNumber > 196 || exemplarSectionNumber < 0 || exemplarSectionNumber > this.exemplarSectionSizeArray[paperNumber].length - 1)
         {
         return -1;
         }
      var numberOfLines = this.numberOfLinesInSection(paperNumber, exemplarSectionNumber);
      if (paperNumber == 0 && exemplarSectionNumber == 12)
         {
         return numberOfLines - 1;
         }
      else
         {
         return numberOfLines;
         }
      }

   this.absNumberOfSublinesInLine = function (paperNumber, lineNumber)
      {
      if (!this.paperNumberAndLineNumberAreValid(paperNumber, lineNumber))
         {
         return -1;
         }
      if (lineNumber == 1 && paperNumber > 0)
         {
         return 2;
         }
      this.expandExemplarDataForPaperWhereNecessary(paperNumber);
      var sublinesCount = this.vabsNumberOfSublinesInLineBookwideArray[paperNumber + ":" + lineNumber];
      return sublinesCount == null ? 0 : sublinesCount;
      }

   this.absNumberOfSublinesInLineBookwideArray = function ()
      {
      var arrayForReturn = new Array();
      for (var sublinesNumber in this.vabsNumberOfSublinesInLineBookwideArray)
         {
         arrayForReturn[sublinesNumber] = this.vabsNumberOfSublinesInLineBookwideArray[sublinesNumber];
         }
      for (sublinesNumber = 1; sublinesNumber <= 197; sublinesNumber++)
         {
         arrayForReturn[sublinesNumber + ":1"] = 2;
         }
      return arrayForReturn;
      }

   this.absSectionNumberOfLine = function (paperNumber, lineNumber)
      {
      var lineNumberExemplarSectionNumber = "";
      if (!this.paperNumberAndLineNumberAreValid(paperNumber, lineNumber))
         {
         return -1;
         }
      this.expandExemplarDataForPaperWhereNecessary(paperNumber);
      lineNumberExemplarSectionNumber = this.sectionNumberOfLine(paperNumber, lineNumber);
      if (paperNumber != 139 || lineNumberExemplarSectionNumber <= 9)
         {
         return lineNumberExemplarSectionNumber;
         }
      else
         {
      return lineNumberExemplarSectionNumber + 1;
         }
      }

   this.absLineNumberInSectionInPaper = function (paperNumber, absSectionNumber, lineNumberInAbsSection)
      {
      this.expandExemplarDataForPaperWhereNecessary(paperNumber);
      var exemplarSectionNumber =  this.absSectionNumberToExemplarSectionNumber(paperNumber, absSectionNumber, lineNumberInAbsSection);
      var baseLineNumber = this.lineNumberInPaperOfSectionTitle(paperNumber, exemplarSectionNumber);
      return parseInt(baseLineNumber) + parseInt(lineNumberInAbsSection);
      }

   this.exemplarSectionNumberToAbsSectionNumber = function (paperNumber, exemplarSectionNumber)
      {
      if (paperNumber < 0 || paperNumber > 196 || exemplarSectionNumber < 0 || exemplarSectionNumber > this.exemplarSectionSizeArray[paperNumber].length - 1)
         {
         return -1;
         }
      return paperNumber == 139 && exemplarSectionNumber > 9 ? exemplarSectionNumber + 1 : exemplarSectionNumber;
      }

   this.absSectionNumberToExemplarSectionNumber = function (paperNumber, absSectionNumber)
      {
      if (paperNumber < 0 || paperNumber > 196 )
         {
         return -1;
         }
      return paperNumber == 139 && absSectionNumber > 9 ? absSectionNumber - 1 : absSectionNumber;
      }

   this.expandAbsDataForPaperWhereNecessary = function (paperNumber)
      {
      if (this.vcurrentAbsPaperNumber != paperNumber)
         {
         this.expandAbsDataForPaper(paperNumber);
         }
      }

   this.lineIsAcknowledgmentHeading = function (paperNumber, lineNumberInPaper)
      {
      if (!this.paperNumberAndLineNumberAreValid(paperNumber, lineNumberInPaper))
         {
         return -1;
         }
      return paperNumber == 0 && lineNumberInPaper == 181;
      }

   this.lineIsPaperTitle = function (lineNumberInPaper)
      {
      return lineNumberInPaper == 1;
      }

   this.lineIsSectionTitle = function (paperNumber, lineNumberInPaper)
      {
      if (!this.paperNumberAndLineNumberAreValid(paperNumber, lineNumberInPaper))
         {
         return -1;
         }
      this.expandExemplarDataForPaperWhereNecessary(paperNumber);
      return this.lineType(paperNumber, lineNumberInPaper) == "sectiontitle";
      }

   this.lineIsSectionTitle = function (paperNumber, lineNumberInPaper)
      {
      if (!this.paperNumberAndLineNumberAreValid(paperNumber, lineNumberInPaper))
         {
         return -1;
         }
      this.expandExemplarDataForPaperWhereNecessary(paperNumber);
      return this.lineType(paperNumber, lineNumberInPaper) == "sectiontitle";
      }

   }

// inherit exemplarMap
   absMap.prototype = new exemplarMap();

//------------------------------------------------
